'use strict';

exports.__esModule = true;
exports.HeatMapDefaultProps = undefined;

var _isEqual2 = require('lodash/isEqual');

var _isEqual3 = _interopRequireDefault(_isEqual2);

var _max2 = require('lodash/max');

var _max3 = _interopRequireDefault(_max2);

var _min2 = require('lodash/min');

var _min3 = _interopRequireDefault(_min2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /*
                                                                                                                                                                                                                                                                   * This file is part of the nivo project.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                   * file that was distributed with this source code.
                                                                                                                                                                                                                                                                   */


var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactMotion = require('react-motion');

var _compose = require('recompose/compose');

var _compose2 = _interopRequireDefault(_compose);

var _defaultProps = require('recompose/defaultProps');

var _defaultProps2 = _interopRequireDefault(_defaultProps);

var _withPropsOnChange = require('recompose/withPropsOnChange');

var _withPropsOnChange2 = _interopRequireDefault(_withPropsOnChange);

var _pure = require('recompose/pure');

var _pure2 = _interopRequireDefault(_pure);

var _d3Scale = require('d3-scale');

var _hocs = require('../../../hocs');

var _colors = require('../../../lib/colors');

var _propertiesConverters = require('../../../lib/propertiesConverters');

var _props = require('../../../props');

var _Container = require('../Container');

var _Container2 = _interopRequireDefault(_Container);

var _SvgWrapper = require('../SvgWrapper');

var _SvgWrapper2 = _interopRequireDefault(_SvgWrapper);

var _Grid = require('../../axes/Grid');

var _Grid2 = _interopRequireDefault(_Grid);

var _Axes = require('../../axes/Axes');

var _Axes2 = _interopRequireDefault(_Axes);

var _HeatMapCellRect = require('./HeatMapCellRect');

var _HeatMapCellRect2 = _interopRequireDefault(_HeatMapCellRect);

var _HeatMapCellCircle = require('./HeatMapCellCircle');

var _HeatMapCellCircle2 = _interopRequireDefault(_HeatMapCellCircle);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var computeX = function computeX(column, cellWidth, padding) {
    return column * cellWidth + cellWidth * 0.5 + padding * column + padding;
};
var computeY = function computeY(row, cellHeight, padding) {
    return row * cellHeight + cellHeight * 0.5 + padding * row + padding;
};

var HeatMap = function HeatMap(_ref) {
    var data = _ref.data,
        getIndex = _ref.getIndex,
        keys = _ref.keys,
        cellWidth = _ref.cellWidth,
        cellHeight = _ref.cellHeight,
        sizeScale = _ref.sizeScale,
        xScale = _ref.xScale,
        yScale = _ref.yScale,
        offsetX = _ref.offsetX,
        offsetY = _ref.offsetY,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        outerWidth = _ref.outerWidth,
        outerHeight = _ref.outerHeight,
        cellShape = _ref.cellShape,
        cellOpacity = _ref.cellOpacity,
        cellBorderWidth = _ref.cellBorderWidth,
        getCellBorderColor = _ref.getCellBorderColor,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        getLabelTextColor = _ref.getLabelTextColor,
        theme = _ref.theme,
        colorScale = _ref.colorScale,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping,
        isInteractive = _ref.isInteractive;

    var Cell = void 0;
    if (cellShape === 'rect') {
        Cell = _HeatMapCellRect2.default;
    } else if (cellShape === 'circle') {
        Cell = _HeatMapCellCircle2.default;
    } else {
        Cell = cellShape;
    }

    var nodes = data.reduce(function (acc, d) {
        keys.forEach(function (key) {
            acc.push({
                key: key + '.' + getIndex(d),
                xKey: key,
                yKey: getIndex(d),
                x: xScale(key),
                y: yScale(getIndex(d)),
                width: sizeScale ? Math.min(sizeScale(d[key]) * cellWidth, cellWidth) : cellWidth,
                height: sizeScale ? Math.min(sizeScale(d[key]) * cellHeight, cellHeight) : cellHeight,
                value: d[key],
                color: colorScale(d[key])
            });
        });

        return acc;
    }, []);

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    var cells = void 0;
    if (animate === true) {
        cells = _react2.default.createElement(
            _reactMotion.TransitionMotion,
            {
                styles: nodes.map(function (node) {
                    return {
                        key: node.key,
                        data: node,
                        style: _extends({
                            x: (0, _reactMotion.spring)(node.x, motionProps),
                            y: (0, _reactMotion.spring)(node.y, motionProps),
                            width: (0, _reactMotion.spring)(node.width, motionProps),
                            height: (0, _reactMotion.spring)(node.height, motionProps)
                        }, (0, _colors.colorMotionSpring)(node.color, motionProps))
                    };
                })
            },
            function (interpolatedStyles) {
                return _react2.default.createElement(
                    'g',
                    null,
                    interpolatedStyles.map(function (_ref2) {
                        var key = _ref2.key,
                            style = _ref2.style,
                            node = _ref2.data;

                        var color = (0, _colors.getInterpolatedColor)(style);

                        return _react2.default.createElement(Cell, {
                            key: key,
                            value: node.value,
                            x: style.x,
                            y: style.y,
                            width: Math.max(style.width, 0),
                            height: Math.max(style.height, 0),
                            color: color,
                            opacity: cellOpacity,
                            borderWidth: cellBorderWidth,
                            borderColor: getCellBorderColor(_extends({}, node, { color: color })),
                            textColor: getLabelTextColor(_extends({}, node, { color: color }))
                        });
                    })
                );
            }
        );
    } else {
        cells = nodes.map(function (node) {
            return _react2.default.createElement(Cell, {
                key: node.key,
                value: node.value,
                x: node.x,
                y: node.y,
                width: node.width,
                height: node.height,
                color: node.color,
                opacity: cellOpacity,
                borderWidth: cellBorderWidth,
                borderColor: getCellBorderColor(node),
                textColor: getLabelTextColor(node)
            });
        });
    }

    return _react2.default.createElement(
        _Container2.default,
        { isInteractive: isInteractive, theme: theme },
        function (_ref3) {
            var showTooltip = _ref3.showTooltip,
                hideTooltip = _ref3.hideTooltip;

            return _react2.default.createElement(
                _SvgWrapper2.default,
                {
                    width: outerWidth,
                    height: outerHeight,
                    margin: Object.assign({}, margin, {
                        top: margin.top + offsetY,
                        left: margin.left + offsetX
                    })
                },
                _react2.default.createElement(_Grid2.default, _extends({
                    theme: theme,
                    width: width - offsetX * 2,
                    height: height - offsetY * 2,
                    xScale: enableGridX ? xScale : null,
                    yScale: enableGridY ? yScale : null
                }, motionProps)),
                _react2.default.createElement(_Axes2.default, _extends({
                    xScale: xScale,
                    yScale: yScale,
                    width: width,
                    height: height,
                    theme: theme,
                    top: axisTop,
                    right: axisRight,
                    bottom: axisBottom,
                    left: axisLeft
                }, motionProps)),
                cells
            );
        }
    );
};

HeatMap.propTypes = {
    // data
    data: _propTypes2.default.arrayOf(_propTypes2.default.object).isRequired,
    indexBy: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]).isRequired,
    getIndex: _propTypes2.default.func.isRequired, // computed
    keys: _propTypes2.default.arrayOf(_propTypes2.default.string).isRequired,

    minValue: _propTypes2.default.oneOfType([_propTypes2.default.oneOf(['auto']), _propTypes2.default.number]).isRequired,
    maxValue: _propTypes2.default.oneOfType([_propTypes2.default.oneOf(['auto']), _propTypes2.default.number]).isRequired,

    forceSquare: _propTypes2.default.bool.isRequired,
    sizeVariation: _propTypes2.default.number.isRequired,
    padding: _propTypes2.default.number.isRequired,

    // cells
    cellShape: _propTypes2.default.oneOfType([_propTypes2.default.oneOf(['rect', 'circle']), _propTypes2.default.func]).isRequired,
    cellOpacity: _propTypes2.default.number.isRequired,
    cellHoverOpacity: _propTypes2.default.number.isRequired,
    cellBorderWidth: _propTypes2.default.number.isRequired,
    cellBorderColor: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]).isRequired,
    getCellBorderColor: _propTypes2.default.func.isRequired, // computed

    // axes & grid
    axisTop: _propTypes2.default.object,
    axisRight: _propTypes2.default.object,
    axisBottom: _propTypes2.default.object,
    axisLeft: _propTypes2.default.object,
    enableGridX: _propTypes2.default.bool.isRequired,
    enableGridY: _propTypes2.default.bool.isRequired,

    // labels
    enableLabels: _propTypes2.default.bool.isRequired,
    labelTextColor: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]).isRequired,
    getLabelTextColor: _propTypes2.default.func.isRequired, // computed

    // theming
    colors: _props.quantizeColorScalePropType.isRequired,
    colorScale: _propTypes2.default.func.isRequired, // computed

    // interactivityP
    isInteractive: _propTypes2.default.bool
};

var HeatMapDefaultProps = exports.HeatMapDefaultProps = {
    indexBy: 'id',

    minValue: 'auto',
    maxValue: 'auto',

    forceSquare: false,
    sizeVariation: 0,
    padding: 0,

    // cells
    cellShape: 'rect',
    cellOpacity: 0.85,
    cellHoverOpacity: 1,
    cellBorderWidth: 0,
    cellBorderColor: 'inherit',

    // axes & grid
    axisTop: {},
    axisLeft: {},
    enableGridX: false,
    enableGridY: false,

    // labels
    enableLabels: true,
    labelTextColor: 'inherit:darker(1.4)',

    // theming
    colors: 'nivo',

    // interactivity
    isInteractive: true
};

HeatMap.defaultProps = HeatMapDefaultProps;

var enhance = (0, _compose2.default)((0, _defaultProps2.default)(HeatMapDefaultProps), (0, _hocs.withTheme)(), (0, _hocs.withDimensions)(), (0, _hocs.withMotion)(), (0, _withPropsOnChange2.default)(['colors'], function (_ref4) {
    var colors = _ref4.colors;
    return {
        colorScale: (0, _colors.guessQuantizeColorScale)(colors)
    };
}), (0, _withPropsOnChange2.default)(['indexBy'], function (_ref5) {
    var indexBy = _ref5.indexBy;
    return {
        getIndex: (0, _propertiesConverters.getAccessorFor)(indexBy)
    };
}), (0, _withPropsOnChange2.default)(['data', 'keys', 'width', 'height', 'padding', 'forceSquare'], function (_ref6) {
    var data = _ref6.data,
        keys = _ref6.keys,
        width = _ref6.width,
        height = _ref6.height,
        padding = _ref6.padding,
        forceSquare = _ref6.forceSquare;

    var columns = keys.length;
    var rows = data.length;

    var cellWidth = Math.max((width - padding * (columns + 1)) / columns, 0);
    var cellHeight = Math.max((height - padding * (rows + 1)) / rows, 0);

    var offsetX = 0;
    var offsetY = 0;
    if (forceSquare === true) {
        var cellSize = Math.min(cellWidth, cellHeight);
        cellWidth = cellSize;
        cellHeight = cellSize;

        offsetX = (width - ((cellWidth + padding) * columns + padding)) / 2;
        offsetY = (height - ((cellHeight + padding) * rows + padding)) / 2;
    }

    return {
        cellWidth: cellWidth,
        cellHeight: cellHeight,
        offsetX: offsetX,
        offsetY: offsetY
    };
}), (0, _withPropsOnChange2.default)(['data', 'getIndex'], function (_ref7) {
    var data = _ref7.data,
        getIndex = _ref7.getIndex;
    return {
        indices: data.map(getIndex)
    };
}), (0, _withPropsOnChange2.default)(function (prev, next) {
    return prev.keys !== next.keys || prev.cellWidth !== next.cellWidth || prev.cellHeight !== next.cellHeight || prev.padding !== next.padding || !(0, _isEqual3.default)(prev.indices, next.indices);
}, function (_ref8) {
    var indices = _ref8.indices,
        keys = _ref8.keys,
        cellWidth = _ref8.cellWidth,
        cellHeight = _ref8.cellHeight,
        padding = _ref8.padding;
    return {
        xScale: (0, _d3Scale.scaleOrdinal)(keys.map(function (key, i) {
            return computeX(i, cellWidth, padding);
        })).domain(keys),
        yScale: (0, _d3Scale.scaleOrdinal)(indices.map(function (d, i) {
            return computeY(i, cellHeight, padding);
        })).domain(indices)
    };
}), (0, _withPropsOnChange2.default)(['data', 'keys', 'minValue', 'maxValue'], function (_ref9) {
    var data = _ref9.data,
        keys = _ref9.keys,
        _minValue = _ref9.minValue,
        _maxValue = _ref9.maxValue;

    var minValue = _minValue;
    var maxValue = _maxValue;
    if (minValue === 'auto' || maxValue === 'auto') {
        var allValues = data.reduce(function (acc, row) {
            return acc.concat(keys.map(function (key) {
                return row[key];
            }));
        }, []);

        if (minValue === 'auto') minValue = (0, _min3.default)(allValues);
        if (maxValue === 'auto') maxValue = (0, _max3.default)(allValues);
    }

    return {
        minValue: Math.min(minValue, maxValue),
        maxValue: Math.max(maxValue, minValue)
    };
}), (0, _withPropsOnChange2.default)(['colorScale', 'minValue', 'maxValue'], function (_ref10) {
    var colorScale = _ref10.colorScale,
        minValue = _ref10.minValue,
        maxValue = _ref10.maxValue;
    return {
        colorScale: colorScale.domain([minValue, maxValue])
    };
}), (0, _withPropsOnChange2.default)(['sizeVariation', 'minValue', 'maxValue'], function (_ref11) {
    var sizeVariation = _ref11.sizeVariation,
        minValue = _ref11.minValue,
        maxValue = _ref11.maxValue;

    var sizeScale = void 0;
    if (sizeVariation > 0) {
        sizeScale = (0, _d3Scale.scaleLinear)().range([1 - sizeVariation, 1]).domain([minValue, maxValue]);
    }

    return { sizeScale: sizeScale };
}), (0, _withPropsOnChange2.default)(['cellBorderColor'], function (_ref12) {
    var cellBorderColor = _ref12.cellBorderColor;
    return {
        getCellBorderColor: (0, _colors.getInheritedColorGenerator)(cellBorderColor)
    };
}), (0, _withPropsOnChange2.default)(['labelTextColor'], function (_ref13) {
    var labelTextColor = _ref13.labelTextColor;
    return {
        getLabelTextColor: (0, _colors.getInheritedColorGenerator)(labelTextColor)
    };
}), _pure2.default);

exports.default = enhance(HeatMap);